<?php

namespace Drupal\salesforce_mfw\Plugin\Block;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Provides a Salesforce MFW block.
 *
 * @Block(
 *  id = "salesforce_mfw",
 *  admin_label = @Translation("Salesforce MFW"),
 * )
 */
class SalesforceMfw extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['warning'] = [
      '#type' => 'markup',
      '#markup' => '<div class="messages messages--warning">' . $this->t('<b>Note:</b> You can add multiple Salesforce MFW blocks to your site, but make sure that only one block is configured to display on any given page(s). Different chat utilities can be assigned to different pages. Configuring multiple blocks to appear on the same page might lead to unpredictable results.') . '</div>',
    ];

    // Initialize form state fields if not already set.
    if ($form_state->get('prechat_fields') === NULL) {
      $form_state->set('prechat_fields', $this->configuration['prechat_fields'] ?? []);
    }

    $form['organization_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Organization id'),
      '#description' => $this->t('Enter the organization of the Salesforce utility'),
      '#default_value' => $this->configuration['organization_id'] ?? '',
      '#required' => TRUE,
    ];

    $form['config_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Config name'),
      '#description' => $this->t('Enter the config name of the Salesforce utility.'),
      '#default_value' => $this->configuration['config_name'] ?? '',
      '#required' => TRUE,
    ];

    $form['site_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site url'),
      '#description' => $this->t('Enter the site url of the Salesforce utility.'),
      '#default_value' => $this->configuration['site_url'] ?? '',
      '#required' => TRUE,
    ];

    $form['snippet_config'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Snippet config url'),
      '#description' => $this->t('Enter the snippet config url of the Salesforce utility.'),
      '#default_value' => $this->configuration['snippet_config'] ?? '',
      '#required' => TRUE,
    ];

    $form['utility_bootstrap'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Utility bootstrap'),
      '#description' => $this->t('Enter the Salesforce utility bootstrap url.'),
      '#default_value' => $this->configuration['utility_bootstrap'] ?? '',
      '#required' => TRUE,
    ];

    $form['language'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Language'),
      '#description' => $this->t('Enter the language of the utility - I.E - en_US'),
      '#default_value' => $this->configuration['language'] ?? '',

    ];

    $form['allow_token_replacement'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Replace tokens'),
      '#description' => $this->t('Checking this box will allow tokens to be replaced if available in default values'),
      '#default_value' => $this->configuration['allow_token_replacement'] ?? FALSE,
    ];
    $form['allow_path_override'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow Path Override'),
      '#description' => $this->t('Checking this box will bypass allowed paths if there is an active chat interaction.'),
      '#default_value' => $this->configuration['allow_path_override'] ?? FALSE,
    ];

    // Visible fields table.
    $form['fields_container'] = [
      '#type' => 'details',
      '#title' => $this->t('Pre-Chat Fields'),
      '#description' => $this->t('This form provides the ability to add default values to either hidden or visible pre-chat fields. For more advanced setup needs, such as setting a default value of a field via a cookie or a user\'s local storage, check out the <a href="@url">API Documentation</a>.', ['@url' => 'https://www.drupal.org/docs/extending-drupal/contributed-modules/contributed-module-documentation/salesforce-mfw']),
      '#open' => TRUE,
    ];

    // Add visible fields to the table.
    $form['fields_container']['prechat_fields'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Name'),
        $this->t('Default value'),
        $this->t('Hidden'),
        $this->t('Editable by user'),
        $this->t('Operations'),
      ],
      '#prefix' => '<div id="visible-fields-wrapper">',
      '#suffix' => '</div>',
      '#empty' => $this->t('No fields added yet.'),
    ];

    // Load existing fields from the form state.
    $fields = $form_state->get('prechat_fields');
    if ($fields) {
      foreach ($fields as $key => $field) {
        $form['fields_container']['prechat_fields'][$key]['name'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Name'),
          '#title_display' => 'invisible',
          '#default_value' => $field['name'],
          '#required' => TRUE,
        ];

        $form['fields_container']['prechat_fields'][$key]['default_value'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Default value'),
          '#title_display' => 'invisible',
          '#default_value' => $field['default_value'] ?? '',
        ];

        $form['fields_container']['prechat_fields'][$key]['hidden'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Hidden'),
          '#title_display' => 'invisible',
          '#default_value' => $field['hidden'] ?? FALSE,
        ];

        $form['fields_container']['prechat_fields'][$key]['editable_by_user'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Editable by user'),
          '#title_display' => 'invisible',
          '#default_value' => $field['editable_by_user'] ?? FALSE,
          '#states' => [
            'disabled' => [
              ':input[name="fields[' . $key . '][hidden]"]' => ['checked' => TRUE],
            ],
          ],
        ];

        $form['fields_container']['prechat_fields'][$key]['operations'] = [
          '#type' => 'submit',
          '#value' => $this->t('Remove'),
          '#name' => 'remove_field_' . $key,
          '#submit' => [[$this, 'removeFieldCallback']],
          '#ajax' => [
            'callback' => [$this, 'updateFieldsTable'],
            'wrapper' => 'visible-fields-wrapper',
          ],
        ];
      }
    }

    $form['fields_container']['add_field'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Field'),
      '#submit' => [[$this, 'addFieldCallback']],
      '#ajax' => [
        'callback' => [$this, 'updateFieldsTable'],
        'wrapper' => 'visible-fields-wrapper',
      ],
    ];

    // Add the token browser link.
    $form['token_browser'] = [
      '#type' => 'link',
      '#title' => $this->t('Browse Available Tokens'),
      '#url' => Url::fromRoute('token.tree'),
      '#attributes' => [
        'class' => ['use-ajax'],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => Json::encode(['width' => 700]),
      ],
    ];

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['allow_path_override'] = $form_state->getValue('allow_path_override');
    $this->configuration['organization_id'] = $form_state->getValue('organization_id');
    $this->configuration['config_name'] = $form_state->getValue('config_name');
    $this->configuration['site_url'] = $form_state->getValue('site_url');
    $this->configuration['snippet_config'] = $form_state->getValue('snippet_config');
    $this->configuration['utility_bootstrap'] = $form_state->getValue('utility_bootstrap');
    $this->configuration['language'] = $form_state->getValue('language');
    $this->configuration['allow_token_replacement'] = $form_state->getValue('allow_token_replacement');
    $this->configuration['prechat_fields'] = $form_state->getValue(['fields_container', 'prechat_fields']) ?: [];

    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build['#attached']['drupalSettings']['salesforce_mfw'][$this->configuration['config_name']]['configuration']['chat_visible'] = TRUE;

    return $build;
  }

  /**
   * Callback to remove a field from visible fields.
   */
  public function removeFieldCallback(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $field_key = str_replace('remove_field_', '', $triggering_element['#name']);
    $fields = $form_state->get('prechat_fields') ?? [];
    unset($fields[$field_key]);
    $form_state->set('prechat_fields', $fields);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Callback to add a new field to visible fields.
   */
  public function addFieldCallback(array &$form, FormStateInterface $form_state) {
    // Retrieve existing fields from form state or initialize to empty array.
    $fields = !empty($form_state->get('prechat_fields')) ? $form_state->get('prechat_fields') : [];
    // Add a new field with default values.
    $fields[] = [
      'name' => '',
      'default_value' => '',
      'hidden' => FALSE,
      'editable_by_user' => FALSE,
    ];

    // Update the form state with the modified fields.
    $form_state->set('prechat_fields', array_values($fields));

    // Rebuild the form to reflect the updated fields.
    $form_state->setRebuild(TRUE);
  }

  /**
   * AJAX callback to update the fields table.
   */
  public function updateFieldsTable(array &$form, FormStateInterface $form_state) {
    return $form['settings']['fields_container']['prechat_fields'];
  }

}
