<?php

namespace Drupal\Tests\salesforce_mfw\Kernel;

use Drupal\block\Entity\Block;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\salesforce_mfw\Access\SalesforceMfwAccessCheck;
use Symfony\Component\Routing\Route;

/**
 * Tests the custom Access check.
 *
 * @group salesforce_mfw
 */
class AccessCheckTest extends KernelTestBase {

  /**
   * The SalesforceMfwAccessCheck instance.
   *
   * @var \Drupal\salesforce_mfw\Access\SalesforceMfwAccessCheck
   */
  protected $accessCheck;

  /**
   * The mocked RouteMatchInterface.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $routeMatch;

  /**
   * The mocked AccountInterface.
   *
   * @var \Drupal\Core\Session\AccountInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $account;

  /**
   * The mocked Route.
   *
   * @var \Symfony\Component\Routing\Route|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $route;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'salesforce_mfw',
    'block',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->routeMatch = $this->createMock(RouteMatchInterface::class);
    $this->account = $this->createMock(AccountInterface::class);
    $this->route = $this->createMock(Route::class);
    $this->accessCheck = new SalesforceMfwAccessCheck();
  }

  /**
   * Tests access for create operation.
   */
  public function testAccessCreateOperation() {
    $this->route->method('getRequirement')
      ->with('_salesforce_mfw_access')
      ->willReturn('create');

    $this->routeMatch->method('getParameter')
      ->with('plugin_id')
      ->willReturn('salesforce_mfw');

    $this->account->method('hasPermission')
      ->with('administer salesforce_mfw blocks')
      ->willReturn(TRUE);

    $result = $this->accessCheck->access($this->routeMatch, $this->account, $this->route);
    $this->assertTrue($result->isAllowed());
  }

  /**
   * Tests access for non-create operations.
   */
  public function testAccessNonCreateOperation() {
    $this->route->method('getRequirement')
      ->with('_salesforce_mfw_access')
      ->willReturn('view');

    $block = $this->createMock(Block::class);
    $block->method('getPluginId')
      ->willReturn('salesforce_mfw');

    $this->routeMatch->method('getParameter')
      ->with('block')
      ->willReturn($block);

    $this->account->method('hasPermission')
      ->with('administer salesforce_mfw blocks')
      ->willReturn(TRUE);

    $result = $this->accessCheck->access($this->routeMatch, $this->account, $this->route);
    $this->assertTrue($result->isAllowed());
  }

  /**
   * Tests access denied.
   */
  public function testAccessDenied() {
    $this->route->method('getRequirement')
      ->with('_salesforce_mfw_access')
      ->willReturn('create');

    $this->routeMatch->method('getParameter')
      ->with('plugin_id')
      ->willReturn('salesforce_mfw');

    $this->account->method('hasPermission')
      ->with('administer salesforce_mfw blocks')
      ->willReturn(FALSE);

    $result = $this->accessCheck->access($this->routeMatch, $this->account, $this->route);
    $this->assertFalse($result->isAllowed());
  }

  /**
   * Tests that non salesforce_mfw plugins return neutral when creating.
   */
  public function testCreateAccessWithDifferentPluginType() {
    $this->route->method('getRequirement')
      ->with('_salesforce_mfw_access')
      ->willReturn('create');

    $this->routeMatch->method('getParameter')
      ->with('plugin_id')
      ->willReturn('example_plugin');

    $result = $this->accessCheck->access($this->routeMatch, $this->account, $this->route);
    $this->assertTrue($result->isNeutral());
  }

  /**
   * Tests that non salesforce_mfw plugins return neutral when not creating.
   */
  public function testNonCreateWithDifferentPluginType() {
    $this->route->method('getRequirement')
      ->with('_salesforce_mfw_access')
      ->willReturn('update');

    $block = $this->createMock(Block::class);
    $block->method('getPluginId')
      ->willReturn('example_plugin');

    $this->routeMatch->method('getParameter')
      ->with('block')
      ->willReturn($block);

    $result = $this->accessCheck->access($this->routeMatch, $this->account, $this->route);
    $this->assertTrue($result->isNeutral());
  }

}
